local addonName = ...
local addon = _G[addonName]
local gui = addon.gui

local defaultHighlightTexture = "Interface\\Buttons\\ButtonHilight-Square";

function gui:BuildGui()
	local L = LibStub("AceLocale-3.0"):GetLocale(addonName, true);

	local backdrop = {
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
		edgeFile = "Interface\\ChatFrame\\ChatFrameBackground",
		tile = false,
		tileSize = 0,
		edgeSize = 1
	};

	--Create the frame and assign it to the main variable
	local frame = CreateFrame("Frame", "StrudelStoreFrame", UIParent, BackdropTemplateMixin and "BackdropTemplate")
	self.mainframe = frame;

	--Create the collapsed button
	local collapsedBtn = self:CreateCollapsedButton();
	self.mainCollapsedBtn = collapsedBtn;

	--Setup main frame
	frame:SetFrameStrata("MEDIUM")
	frame:SetBackdrop(backdrop);
	frame:SetBackdropColor(0.1, 0.1, 0.1, 0.6);
	frame:SetPoint("CENTER");
	--Register drag
	frame:SetClampedToScreen(true);
	frame:EnableMouse(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetMovable(true)
	frame:SetScript("OnHide", function(self)
		if ( self.isMoving ) then
			self:StopMovingOrSizing();
			self.isMoving = false;
		end
	end);

	--Header
	frame.titleFrame = CreateFrame("Frame", "$parentTitle", frame);
	frame.titleFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, -5);

	frame.titleFrame.Text = frame.titleFrame:CreateFontString("$parentText", "OVERLAY", "GameToolTipText");
	frame.titleFrame.Text:SetAllPoints(frame.titleFrame);
	frame.titleFrame.Text:SetText(addonName);

	frame.titleFrame:SetSize(frame.titleFrame.Text:GetStringWidth(),frame.titleFrame.Text:GetStringHeight()); --Resize titleFrame to fit text

	--Header Drag code
	frame:SetScript("OnDragStart", function(self)
		self:StartMoving();
		self.isMoving = true;
	end);

	frame:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing();
		self.isMoving = false;

		--Set anchor point
		local anchorPoint = addon.db.char.display.collapseAnchor;
		if not anchorPoint then
			anchorPoint = "TOPRIGHT";
		end
		gui.mainCollapsedBtn:ClearAllPoints();
		gui.mainCollapsedBtn:SetPoint(anchorPoint, self, anchorPoint);
	end);

	--Create the close button
	frame.closeStrudelButton = CreateFrame("Button", "$parentCloseBtn", frame, "SecureHandlerClickTemplate");
	frame.closeStrudelButton:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -5, -5);
	frame.closeStrudelButton:SetWidth(10);
	frame.closeStrudelButton:SetHeight(10);
	frame.closeStrudelButton:SetNormalTexture("Interface\\BUTTONS\\UI-MinusButton-Up");
	frame.closeStrudelButton:SetPushedTexture("Interface\\BUTTONS\\UI-MinusButton-Down");
	frame.closeStrudelButton:SetHighlightTexture(defaultHighlightTexture);
	frame.closeStrudelButton:SetAttribute("_onclick", [=[
		local frameRef = self:GetFrameRef("strudelFrame");
		frameRef:Hide();

		local btnRef = self:GetFrameRef("collapsedStrudelBtn");
		if not btnRef:IsShown() then
			btnRef:Show();
		end
	]=]);
	frame.closeStrudelButton:SetFrameRef("collapsedStrudelBtn", collapsedBtn);
	frame.closeStrudelButton:SetFrameRef("strudelFrame", frame);

	--Create the config button
	frame.configButton = CreateFrame("Button", "$parentConfigBtn", frame);
	frame.configButton:SetPoint("TOPRIGHT", frame.closeStrudelButton, "TOPLEFT", -5, 0);
	frame.configButton:SetWidth(10);
	frame.configButton:SetHeight(10);
	frame.configButton:SetNormalTexture("Interface\\BUTTONS\\UI-OptionsButton");
	frame.configButton:SetPushedTexture("Interface\\BUTTONS\\UI-OptionsButton");
	frame.configButton:SetHighlightTexture(defaultHighlightTexture);
	frame.configButton:SetScript("OnClick", function(self, button)
		--Call the method twice to prevent bug causing the window to not open
		InterfaceOptionsFrame_OpenToCategory(addonName);
		InterfaceOptionsFrame_OpenToCategory(addonName);
	end);

	--Create the tabbar
	frame.tabBar = CreateFrame("Frame", "$parentTabBar", frame);
	frame.tabBar.SetActiveTabIndex = function(tabIndex)
		frame.tabBar.activetabindex = tabIndex;
		addon.db.char.display.selectedTab = tabIndex;
		for i, tab in ipairs(frame.tabs) do
			if i == tabIndex then
				tab:Show();
			else
				tab:Hide();
			end
		end
		self:SizeMainFrame();
	end;
	--frame.tabBar:SetBackdrop(backdrop);
	--frame.tabBar:SetBackdropColor(0.3, 1, 0, 0.6);
	frame.tabBar:SetSize(100,30)
	frame.tabBar:SetPoint("TOPRIGHT", frame, "BOTTOMRIGHT", 0,0);

	if not addon.isRunningMainline then
		frame.tabBar.conjureTabButton = CreateFrame("Button", "$parentConjure", frame.tabBar);
		frame.tabBar.conjureTabButton:SetWidth(30);
		frame.tabBar.conjureTabButton:SetHeight(30);
		frame.tabBar.conjureTabButton:SetPoint("TOPRIGHT", frame.tabBar, "TOPRIGHT", 0,0);
		frame.tabBar.conjureTabButton:SetNormalTexture("Interface\\Icons\\INV_Drink_06");
		frame.tabBar.conjureTabButton:SetHighlightTexture(defaultHighlightTexture);
		frame.tabBar.conjureTabButton:SetScript("onClick", function()
			frame.tabBar.SetActiveTabIndex(1);
		end);
		frame.tabBar.conjureTabButton:SetScript("OnEnter", function() 
			GameTooltip:SetOwner(frame.tabBar.conjureTabButton, "ANCHOR_BOTTOMRIGHT");
			GameTooltip:SetText(L["ConjureFrameTitle"], nil, nil, nil, 0.5, 1); 
		end);
		frame.tabBar.conjureTabButton:SetScript("OnLeave", function() 
			GameTooltip:Hide();
		end);
	end

	frame.tabBar.portalTabButton = CreateFrame("Button", "$parentPortal", frame.tabBar);
	frame.tabBar.portalTabButton:SetWidth(30);
	frame.tabBar.portalTabButton:SetHeight(30);
	frame.tabBar.portalTabButton:SetNormalTexture("Interface\\Icons\\Spell_Arcane_PortalStormWind");
	frame.tabBar.portalTabButton:SetHighlightTexture(defaultHighlightTexture);
	frame.tabBar.portalTabButton:SetScript("onClick", function()
		frame.tabBar.SetActiveTabIndex(2);
	end);
	frame.tabBar.portalTabButton:SetScript("OnEnter", function() 
		GameTooltip:SetOwner(frame.tabBar.portalTabButton, "ANCHOR_BOTTOMRIGHT");
		GameTooltip:SetText(L["PortalFrameTitle"], nil, nil, nil, 0.5, 1); 
	end);
	frame.tabBar.portalTabButton:SetScript("OnLeave", function() 
		GameTooltip:Hide();
	end);

	--If addon is running in classic/BCC/Wrath align the button to the existing conjure button.
	--Otherwise align it to the right (since conjure button isn't created when running retail)
	if not addon.isRunningMainline then
		frame.tabBar.portalTabButton:SetPoint("TOPRIGHT", frame.tabBar.conjureTabButton, "TOPLEFT", -5,0);
	else
		frame.tabBar.portalTabButton:SetPoint("TOPRIGHT", frame.tabBar, "TOPRIGHT", 0,0);
	end

	frame.tabBar.teleportTabButton = CreateFrame("Button", "$parentTeleport", frame.tabBar);
	frame.tabBar.teleportTabButton:SetWidth(30);
	frame.tabBar.teleportTabButton:SetHeight(30);
	frame.tabBar.teleportTabButton:SetPoint("TOPRIGHT", frame.tabBar.portalTabButton, "TOPLEFT", -5,0);
	frame.tabBar.teleportTabButton:SetNormalTexture("Interface\\Icons\\Spell_Arcane_TeleportStormWind");
	frame.tabBar.teleportTabButton:SetHighlightTexture(defaultHighlightTexture);
	frame.tabBar.teleportTabButton:SetScript("onClick", function()
		frame.tabBar.SetActiveTabIndex(3);
	end);
	frame.tabBar.teleportTabButton:SetScript("OnEnter", function() 
		GameTooltip:SetOwner(frame.tabBar.teleportTabButton, "ANCHOR_BOTTOMRIGHT");
		GameTooltip:SetText(L["TeleportFrameTitle"], nil, nil, nil, 0.5, 1); 
	end);
	frame.tabBar.teleportTabButton:SetScript("OnLeave", function() 
		GameTooltip:Hide();
	end);
end

function gui:CreateCollapsedButton()
	--Create the collapsed frame
	local collapsedBtn = CreateFrame("Button", "StrudelStoreCollapsedBtn", UIParent, "SecureHandlerClickTemplate");
	collapsedBtn:SetPoint("CENTER");
	collapsedBtn:SetWidth(20);
	collapsedBtn:SetHeight(20);
	collapsedBtn:SetFrameStrata("MEDIUM")
	collapsedBtn:SetClampedToScreen(true);
	collapsedBtn:EnableMouse(true)
	collapsedBtn:RegisterForDrag("LeftButton")
	collapsedBtn:SetMovable(true)
	collapsedBtn:SetScript("OnHide", function(self)
		if ( self.isMoving ) then
			self:StopMovingOrSizing();
			self.isMoving = false;
		end
	end);

	local icon = "Interface\\Icons\\Inv_misc_food_73cinnamonroll";
	if not addon.isRunningClassicEra then
		icon = "Interface\\Icons\\Ability_Mage_ConjureFoodRank10";
	end

	local btnTexture = collapsedBtn:CreateTexture(nil,"OVERLAY",nil,-6)
	btnTexture:SetTexture(icon)
	btnTexture:SetAllPoints(collapsedBtn) --make texture same size as button

	collapsedBtn:SetNormalTexture(btnTexture);
	collapsedBtn:SetHighlightTexture(defaultHighlightTexture)
	collapsedBtn:SetAttribute("_onclick", [=[
		mainFrame = self:GetFrameRef("mainFrame");
		if not mainFrame:IsShown() then
			mainFrame:Show();
		end
		self:Hide();
	]=]);
	collapsedBtn:SetFrameRef("mainFrame", self.mainframe);

	--Header Drag code
	collapsedBtn:SetScript("OnDragStart", function(self)
		self:StartMoving();
		self.isMoving = true;
	end);

	collapsedBtn:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing();
		self.isMoving = false;

		--Set anchor point
		local anchorPoint = addon.db.char.display.collapseAnchor;
		gui.mainframe:ClearAllPoints();
		gui.mainframe:SetPoint(anchorPoint, self, anchorPoint);
	end);

	collapsedBtn:Show();
	return collapsedBtn;
end

function gui:ReanchorFrames(self)
	--Set anchor point
	local anchorPoint = addon.db.char.display.collapseAnchor;

	if gui.mainframe:IsShown() then
		gui.mainCollapsedBtn:ClearAllPoints();
		gui.mainCollapsedBtn:SetPoint(anchorPoint, gui.mainframe, anchorPoint);
	elseif gui.mainCollapsedBtn:IsShown() then
		gui.mainframe:ClearAllPoints();
		gui.mainframe:SetPoint(anchorPoint, gui.mainCollapsedBtn, anchorPoint);
	end
end

function gui:PopulateTabs()
	local frame = self.mainframe;

	--Create the buttons for the spells
	frame.tabs = {};
	frame.tabs[1] = self:DrawButtonFrame(frame, "$parentButtonFrameConjure", addon.spells.conjuringSpells);
	frame.tabs[2] = self:DrawButtonFrame(frame, "$parentButtonFramePortal", addon.spells.portalSpells);
	frame.tabs[3] = self:DrawButtonFrame(frame, "$parentButtonFrameTeleport", addon.spells.teleportSpells);

	--Set frame to first tab
	frame.tabBar.SetActiveTabIndex(addon.db.char.display.selectedTab);
end

function gui:SizeMainFrame()
	local frame = self.mainframe;
	--Sizing vars
	local frameMargin = 5;
	local iconoffsetX = addon.db.char.display.spellOffsetX;
	local iconoffsetY = addon.db.char.display.spellOffsetY;
	local maxPerRow = addon.db.char.display.maxInRow;

	local activeTab = frame.tabs[frame.tabBar.activetabindex];
	local buttons = activeTab.spellButtons;
	if table.getn(buttons) > 0 then
		for i, btn in ipairs(buttons) do
			btn:ClearAllPoints();
			--Grab previous button and button on previous row
			local prevButton = buttons [i - 1];
			local prevRowButton = buttons [i - maxPerRow];
			if prevButton == nil then
				--If no buttons exist yet, set initial alignment
				btn:SetPoint("TOPLEFT",0,0);
			else
				if prevRowButton == nil then
					--Align this button to the initial row
					btn:SetPoint("TOPLEFT", prevButton, "TOPRIGHT", iconoffsetX, 0);
					btn:SetPoint("BOTTOMLEFT", prevButton, "BOTTOMRIGHT", iconoffsetX, 0);
				else
					--Align this button to the bottom of the button on the previous row
					btn:SetPoint("TOPLEFT", prevRowButton, "BOTTOMLEFT", 0, -iconoffsetY);
					btn:SetPoint("TOPRIGHT", prevRowButton, "BOTTOMRIGHT", 0, -iconoffsetY);
				end
			end
		end
		local totalButtonCount = table.getn(buttons);
		local totalRowCount = math.ceil(totalButtonCount / maxPerRow);
		local totalColumnCount = maxPerRow < totalButtonCount and maxPerRow or totalButtonCount;

		local titleMargin = frame.titleFrame:GetHeight();
		local totalWidth = (totalColumnCount * (buttons[1]:GetWidth() + iconoffsetX)) + (frameMargin * 2);
		local totalHeight = (totalRowCount * (buttons[1]:GetHeight() + iconoffsetY)) + frameMargin + titleMargin + 5;

		frame:SetSize(totalWidth, totalHeight);

	else
		--If there are no buttons, just give the mainframe a standard size
		frame:SetSize(100, 100);
	end
	activeTab:SetPoint("TOPLEFT", frame.titleFrame, "BOTTOMLEFT", 0,0);
	activeTab:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -frameMargin, frameMargin);
end

function gui:DrawButtonFrame(frame, frameId, spellTable)
	local buttonFrame = CreateFrame("Frame", frameId, frame);
	--buttonFrame:SetBackdrop(backdrop);
	--buttonFrame:SetBackdropColor(1, 1, 0.1, 0.6);
	buttonFrame.spellButtons = {};
	for i, spellId in ipairs(spellTable) do
		local name, rank, icon, castTime, minRange, maxRange = GetSpellInfo(spellId);
		--If spell does exist, create a button for it
		if name then
			local rankNumber = addon.spells.GetSpellRank(spellId);
			local btn = CreateFrame("Button", "StrudelSpellButton" .. spellId, buttonFrame, "SecureActionButtonTemplate");
			btn.buttonSpell = spellId --assign spell id to button (for later reference)
			btn:RegisterForClicks("AnyUp", "AnyDown"); --Needed due to bug in 10.0.2 and up
			btn:SetAttribute("type", "spell");
			if not rankNumber then
				btn:SetAttribute("spell", name);
			else
				local spellWithRank = name .. "(Rank ".. rankNumber ..")";
				btn:SetAttribute("spell", spellWithRank);
			end
			btn:SetAttribute("target", "player");
			btn:SetWidth(35);
			btn:SetHeight(35);

			btn:SetScript("OnEnter", function() 
				GameTooltip:SetOwner(btn, "ANCHOR_TOPRIGHT");
				GameTooltip:SetText(name, nil, nil, nil, 0.5, 1); 
			end);
			btn:SetScript("OnLeave", function() 
				GameTooltip:Hide();
			end);

			local btnTexture = btn:CreateTexture(nil,"OVERLAY",nil,-6)
			btnTexture:SetTexture(icon)
			btnTexture:SetAllPoints(btn) --make texture same size as button
			local btnDisabledTexture = btn:CreateTexture(nil,"OVERLAY",nil,-6)
			btnDisabledTexture:SetTexture(icon)
			btnDisabledTexture:SetAlpha(0.3)
			btnDisabledTexture:SetAllPoints(btn) --make texture same size as button

			btn:SetNormalTexture(btnTexture);
			btn:SetDisabledTexture(btnDisabledTexture);
			btn:SetHighlightTexture(defaultHighlightTexture)

			--Header
			btn.reagentCounterFrame = CreateFrame("Frame", "$parentReagentCount", btn);
			btn.reagentCounterFrame:SetPoint("BOTTOMRIGHT", btn, -1, 1);
			btn.reagentCounterFrame:SetSize(17,14);

			btn.reagentCounterFrame.Text = btn.reagentCounterFrame:CreateFontString("$parentText", "OVERLAY", "NumberFontNormal");
			btn.reagentCounterFrame.Text:SetAllPoints(btn.reagentCounterFrame);

			self:UpdateButton(btn);

			table.insert(buttonFrame.spellButtons, btn);
		end
	end
	return buttonFrame;
end

function gui:UpdateButton(btn)
	local spellId = btn.buttonSpell
	if IsConsumableSpell(spellId) and not addon.db.char.display.hideReagentCount then
		local spellCount = GetSpellCount(spellId)
		btn.reagentCounterFrame.Text:SetText(spellCount)
	else 
		btn.reagentCounterFrame.Text:SetText("")
	end

	local isPlayerSpell = IsPlayerSpell(spellId)
	local isUsable, noMana = IsUsableSpell(spellId)
	btn:SetEnabled(isPlayerSpell and isUsable and not noMana);
end

function gui:UpdateAllTabs()
	local frame = self.mainframe
	if (frame.tabs ~= nil) then
		for tabIndex, tab in ipairs(frame.tabs) do
			for buttonIndex, btn in ipairs(tab.spellButtons) do
				self:UpdateButton(btn)
			end
		end
	end
end

--Show/Hide the main window
function gui:ToggleMainWindow()
	if self.mainframe:IsVisible() then
		self.mainframe:Hide();
	else
		self.mainframe:Show();
		--[[if not UnitAffectingCombat("player") then
			self.mainframe:Show();
		else
			local L = LibStub("AceLocale-3.0"):GetLocale(addonName, true);
			addon:Print(L["CombatError"]);
		end]]--
	end
end